#!/usr/bin/env python3
import os
import logging
from flask import Flask
from flask_restful import Api
import sentry_sdk
from sentry_sdk.integrations.flask import FlaskIntegration
from sentry_sdk.integrations.sqlalchemy import SqlalchemyIntegration
from healthcheck import HealthCheck

from jwtman import jwtman
from marshm import ma
from fred import flaskred
from config import SENTRY_DSN, JWT_SECRET_KEY, RELEASEMODE, RELEASE_ID, PORT, DEBUG, REDIS_URL
from errorhandlers import register_all_error_handlers
from resources import LoginApi

"""
Main Flask RESTful API
"""

__author__ = "@tormakris"
__copyright__ = "Copyright 2020, onSpot Team"
__module_name__ = "app"
__version__text__ = "1"

if SENTRY_DSN:
    sentry_sdk.init(
        dsn=SENTRY_DSN,
        integrations=[FlaskIntegration(), SqlalchemyIntegration()],
        traces_sample_rate=1.0,
        send_default_pii=True,
        release=RELEASE_ID,
        environment=RELEASEMODE,
        _experiments={"auto_enabling_integrations": True}
    )

app = Flask(__name__)
app.config['JWT_SECRET_KEY'] = JWT_SECRET_KEY
app.config['REDIS_URL'] = REDIS_URL
api = Api(app)
health = HealthCheck()
ma.init_app(app)
flaskred.init_app(app)
jwtman.init_app(app)

formatter = logging.Formatter(
    fmt="%(asctime)s - %(levelname)s - %(module)s - %(message)s"
)

handler = logging.StreamHandler()
handler.setFormatter(formatter)

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
logger.addHandler(handler)

api.add_resource(LogoffApi, '/api/auth/logoff')
api.add_resource(LoginApi, '/api/auth/login')
api.add_resource(MeApi, '/api/auth/me')
api.add_resource(ListsApi, '/api/lists')
api.add_resource(SingleListApi, '/api/lists/<listid>')
api.add_resource(TrackApi, '/api/lists/<listid>/<trackid>')

app.add_url_rule("/healthz", "healthcheck", view_func=lambda: health.run())

register_all_error_handlers(app)


if __name__ == "__main__":
    app.run(
        debug=bool(DEBUG),
        host="0.0.0.0",
        port=int(PORT),
    )